

#pragma once

#include "OLED.h"
#include "SendUBX.h"

namespace GPSSetup
{

    bool sent = true;
    bool Setup(){
      // This UBX command sets the Navigation Mode to Stationary which is optimal for accuracy of Time Pulse
      static uint8_t UBXNav[] PROGMEM = {
      0xB5, 0x62, 0x06, 0x24, 0x24, 0x00, 0xFF, 0xFF, 0x02, 0x03, 0x00, 0x00, 0x00, 0x00, 0x10, 0x27, 0x00, 0x00, 0x05,
      0x00, 0xFA, 0x00, 0xFA, 0x00, 0x64, 0x00, 0x2C, 0x01, 0x00, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x4E, 0x60 }; 
      // This UBX command sets the TimePulse to 1Hz unlocked, 100KHz Locked, both 50% duty cycle
      static uint8_t UBXTP[] PROGMEM = {
      0xB5, 0x62, 0x06, 0x31, 0x20, 0x00, 0x00, 0x01, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xA0,
      0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0xEF, 0x00, 0x00, 0x00,
      0xA1, 0xBA };    
      // get rid of all normal messages except GGA
      // Switching off NMEA GLL
      static uint8_t UBXGLL[] PROGMEM = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x2B };
      // Switching off NMEA GSA
      static uint8_t UBXGSA[] PROGMEM = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x32 };
      // Switching off NMEA GSV
      static  uint8_t UBXGSV[] PROGMEM = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x39 };
      // Switching off NMEA RMC
      static uint8_t UBXRMC[] PROGMEM = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x40 };
      // Switching off NMEA VTG
      static uint8_t UBXVTG[] PROGMEM = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x05, 0x47 };

      // an array of the arrays!
      static uint8_t *UBXdata[] PROGMEM = {UBXNav, UBXTP, UBXGLL, UBXGSA, UBXGSV, UBXRMC, UBXVTG};

      // need the size of each array 
      static size_t sizes[] PROGMEM = {sizeof(UBXNav), sizeof(UBXTP), sizeof(UBXGLL), sizeof(UBXGSA), sizeof(UBXGSV), sizeof(UBXRMC), sizeof(UBXVTG)};

      // now send to GPS and check acknowledged, there are 7 arrays to send.
      for (int x = 0; x < 7; x++){
          debug("Sending %d \n", x);

          sent = SendUBX::sendUBXBuf(UBXdata[x],sizes[x]);  // send the data to the GPS,
          if (!sent) {                                      // will try 5 times and wait a second for a reply
              return false;                                 // send failed so tell user
          } 
          // increment the bar graph
          OLED::fill(x + 1); 
          delay(100);  // a little delay so you can see the bar graph working         
        }
        return true;
    }     
}